@@ -269,7 +269,7 @@ t_fcgi_server *find_fcgi_server(t_fcgi_server *fcgi_server, char *id) {
 
 pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 	int post_pipe[2], html_pipe[2], error_pipe[2], i;
-	char *slash, *run[7], cgi_time[16];
+	char *pos, slash = '/', *run[7], cgi_time[16];
 	pid_t cgi_pid;
 
 	do {
@@ -316,12 +316,19 @@ pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 
 			set_environment(session, NULL);
 
-			if ((slash = strrchr(session->file_on_disk, '/')) != NULL) {
-				*slash = '\0';
+			pos = strrchr(session->file_on_disk, slash);
+#ifdef CYGWIN
+			if ((pos == NULL) && (session->config->platform == windows)) {
+				slash = '\\';
+				pos = strrchr(session->file_on_disk, slash);
+			}
+#endif
+			if (pos != NULL) {
+				*pos = '\0';
 				if (chdir(session->file_on_disk) == -1) {
-					log_file_error(session, session->file_on_disk, "couldn't change to directory");
+					log_file_error(session, session->file_on_disk, "couldn't change to CGI directory");
 				}
-				*slash = '/';
+				*pos = slash;
 			}
 
 			i = 0;
@@ -372,12 +379,6 @@ pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 			cgi_info->from_cgi = html_pipe[0];
 			cgi_info->cgi_error = error_pipe[0];
 
-			if (cgi_info->from_cgi > cgi_info->cgi_error) {
-				cgi_info->highest_fd = cgi_info->from_cgi + 1;
-			} else {
-				cgi_info->highest_fd = cgi_info->cgi_error + 1;
-			}
-
 			/* Send POST data to CGI program.
 			 */
 			if ((session->body != NULL) && (session->content_length > 0)) {
@@ -394,7 +395,7 @@ pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 }
 
 t_cgi_result read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
-	int bytes_read, result = cgi_OKE;
+	int bytes_read, result = cgi_OKE, highest_fd = -1;
 	struct timeval select_timeout;
 	fd_set read_fds;
 	bool read_again;
@@ -403,68 +404,90 @@ t_cgi_result read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 		read_again = false;
 
 		FD_ZERO(&read_fds);
-		FD_SET(cgi_info->from_cgi, &read_fds);
-		FD_SET(cgi_info->cgi_error, &read_fds);
+		if (cgi_info->from_cgi != -1) {
+			FD_SET(cgi_info->from_cgi, &read_fds);
+			if (cgi_info->from_cgi > highest_fd) {
+				highest_fd = cgi_info->from_cgi;
+			}
+		}
+		if (cgi_info->cgi_error != -1) {
+			FD_SET(cgi_info->cgi_error, &read_fds);
+			if (cgi_info->cgi_error > highest_fd) {
+				highest_fd = cgi_info->cgi_error;
+			}
+		}
 
 		select_timeout.tv_sec = 1;
 		select_timeout.tv_usec = 0;
-		switch (select(cgi_info->highest_fd, &read_fds, NULL, NULL, &select_timeout)) {
+		switch (select(highest_fd + 1, &read_fds, NULL, NULL, &select_timeout)) {
 			case -1:
 				if (errno != EINTR) {
 					return cgi_ERROR;
-				} else {
-					read_again = true;
 				}
+				read_again = true;
+				break;
 			case 0:
 				if (session->force_quit) {
 					return cgi_FORCE_QUIT;
 				} else if (time(NULL) > cgi_info->deadline) {
 					return cgi_TIMEOUT;
-				} else {
-					read_again = true;
 				}
+				read_again = true;
+				break;
 		}
 	} while (read_again);
 
 	/* Read CGI output
 	 */
-	if (FD_ISSET(cgi_info->from_cgi, &read_fds)) do {
-		read_again = false;
+	if (cgi_info->from_cgi != -1) {
+		if (FD_ISSET(cgi_info->from_cgi, &read_fds)) do {
+			read_again = false;
 
-		bytes_read = read(cgi_info->from_cgi, cgi_info->input_buffer + cgi_info->input_len, cgi_info->input_buffer_size - cgi_info->input_len);
-		if (bytes_read == -1) {
-			if (errno != EINTR) {
-				return cgi_ERROR;
-			} else {
-				read_again = true;
+			bytes_read = read(cgi_info->from_cgi, cgi_info->input_buffer + cgi_info->input_len, cgi_info->input_buffer_size - cgi_info->input_len);
+			switch (bytes_read) {
+				case -1:
+					if (errno != EINTR) {
+						return cgi_ERROR;
+					}
+					read_again = true;
+					break;
+				case 0:
+					close(cgi_info->from_cgi);
+					cgi_info->from_cgi = -1;
+					break;
+				default:
+					cgi_info->input_len += bytes_read;
 			}
-		} else if (bytes_read == 0) {
-			result = cgi_END_OF_DATA;
-		} else if (bytes_read > 0) {
-			cgi_info->input_len += bytes_read;
-		}
-	} while (read_again);
+		} while (read_again);
+	}
 
 	/* Read CGI error output
 	 */
-	if (FD_ISSET(cgi_info->cgi_error, &read_fds)) do {
-		read_again = false;
+	if (cgi_info->cgi_error != -1) {
+		if (FD_ISSET(cgi_info->cgi_error, &read_fds)) do {
+			read_again = false;
 
-		bytes_read = read(cgi_info->cgi_error, cgi_info->error_buffer + cgi_info->error_len, cgi_info->error_buffer_size - cgi_info->error_len);
-		if (bytes_read == -1) {
-			if (errno != EINTR) {
-				return cgi_ERROR;
-			} else {
-				read_again = true;
+			bytes_read = read(cgi_info->cgi_error, cgi_info->error_buffer + cgi_info->error_len, cgi_info->error_buffer_size - cgi_info->error_len);
+			switch (bytes_read) {
+				case -1:
+					if (errno != EINTR) {
+						return cgi_ERROR;
+					}
+					read_again = true;
+					break;
+				case 0:
+					close(cgi_info->cgi_error);
+					cgi_info->cgi_error = -1;
+					break;
+				default:
+					cgi_info->error_len += bytes_read;
 			}
-/*
-		} else if (bytes_read == 0) {
-			result = cgi_END_OF_DATA;
-*/
-		} else if (bytes_read > 0) {
-			cgi_info->error_len += bytes_read;
-		}
-	} while (read_again);
+		} while (read_again);
+	}
+
+	if ((cgi_info->from_cgi == -1) && (cgi_info->cgi_error == -1)) {
+		result = cgi_END_OF_DATA;
+	}
 
 	return result;
 }
@@ -548,7 +571,7 @@ int send_fcgi_request(t_session *session, int sock) {
 }
 
 static t_cgi_result read_fcgi_socket(t_session *session, t_cgi_info *cgi_info, char *buffer, int size) {
-	int bytes_read, result = cgi_ERROR;
+	int bytes_read;
 	struct timeval select_timeout;
 	fd_set read_fds;
 	bool read_again;
@@ -563,32 +586,33 @@ static t_cgi_result read_fcgi_socket(t_session *session, t_cgi_info *cgi_info, c
 		select_timeout.tv_usec = 0;
 		switch (select(cgi_info->from_cgi + 1, &read_fds, NULL, NULL, &select_timeout)) {
 			case -1:
+				if (errno == EINTR) {
+					read_again = true;
+				}
 				break;
 			case 0:
 				if (session->force_quit) {
-					result = cgi_FORCE_QUIT;
+					return cgi_FORCE_QUIT;
 				} else if (time(NULL) > cgi_info->deadline) {
-					result = cgi_TIMEOUT;
-				} else {
-					read_again = true;
+					return cgi_TIMEOUT;
 				}
+				read_again = true;
 				break;
 			default:
 				do {
 					read_again = false;
 					if ((bytes_read = read(cgi_info->from_cgi, buffer, size)) == -1) {
 						if (errno != EINTR) {
-							return cgi_ERROR;
-						} else {
-							read_again = true;
+							break;
 						}
+						read_again = true;
 					}
 				} while (read_again);
-				result = bytes_read;
+				return bytes_read;
 		}
 	} while (read_again);
 
-	return result;
+	return cgi_ERROR;
 }
 
 t_cgi_result read_from_fcgi_server(t_session *session, t_cgi_info *cgi_info) {
@@ -610,12 +634,7 @@ t_cgi_result read_from_fcgi_server(t_session *session, t_cgi_info *cgi_info) {
 				case cgi_FORCE_QUIT:
 					return cgi_FORCE_QUIT;
 				case cgi_ERROR:
-					if (errno != EINTR) {
-						return cgi_ERROR;
-					} else {
-						read_again = true;
-					}
-					break;
+					return cgi_ERROR;
 				case 0:
 					return cgi_END_OF_DATA;
 				default:
@@ -668,12 +687,7 @@ t_cgi_result read_from_fcgi_server(t_session *session, t_cgi_info *cgi_info) {
 				case cgi_FORCE_QUIT:
 					return cgi_FORCE_QUIT;
 				case cgi_ERROR:
-					if (errno != EINTR) {
-						return cgi_ERROR;
-					} else {
-						read_again = true;
-					}
-					break;
+					return cgi_ERROR;
 				case 0:
 					return cgi_END_OF_DATA;
 				default:
