@@ -190,6 +190,32 @@ int remove_client(t_session *session, bool free_session) {
 	}
 }
 
+/* Check if a write has blocked for too long
+ */
+/*
+void check_write_blocks_on_socket(int max_write_block_time) {
+	t_client *client;
+	int i;
+
+	for (i = 0; i < 256; i++) {
+		pthread_mutex_lock(&client_mutex[i]);
+
+		client = client_list[i];
+		while (client != NULL) {
+			if (client->session->socket_open && (client->session->send_timer >= 0)) {
+				if ((client->session->send_timer)++ == max_write_block_time) {
+					close_socket(client->session);
+				}
+			}
+
+			client = client->next;
+		}
+	
+		pthread_mutex_unlock(&client_mutex[i]);
+	}
+}
+*/
+
 /* Check whether to allow or deny a new connection.
  */
 int connection_allowed(t_ip_addr *ip, int maxperip, int maxtotal) {
@@ -490,6 +516,8 @@ int unban_ip(t_ip_addr *ip) {
 	return result;
 }
 
+/* Remember a client that sent a wrong password for HTTP authentication
+ */
 int register_wrong_password(t_session *session) {
 	t_ipcounterlist *item;
 
