@@ -220,17 +220,9 @@ int handle_error(t_session *session, int error_code) {
 		memcpy(session->file_on_disk, session->host->website_root, length + 1);
 		strcpy(session->file_on_disk + length, error_handler);
 
-		/* Determine the extension of the error handler
-		 */
-		if ((session->extension = strrchr(session->file_on_disk, '.')) != NULL) {
-			(session->extension)++;
-		} else {
-			session->extension = session->config->no_extension_as;
+		if (get_target_extension(session) == -1) {
+			return 500;
 		}
-#ifdef CYGWIN
-		strlower(session->extension);
-#endif
-
 		session->cgi_type = no_cgi;
 		check_target_is_cgi(session);
 
@@ -814,8 +806,10 @@ int serve_client(t_session *session) {
 #endif
 
 	if ((result = fetch_request(session)) != 200) {
+		session->request_method = GET;
 		return result;
 	} else if ((result = parse_request(session, session->header_length + session->content_length)) != 200) {
+		session->request_method = GET;
 		return result;
 	}
 
@@ -1000,12 +994,8 @@ int serve_client(t_session *session) {
 		return 301;
 	}
 
-	/* Determine the extension of the requested file
-	 */
-	if ((session->extension = strrchr(session->file_on_disk, '.')) != NULL) {
-		(session->extension)++;
-	} else {
-		session->extension = session->config->no_extension_as;
+	if (get_target_extension(session) == -1) {
+		return 500;
 	}
 
 	if ((session->request_method != PUT) && (session->request_method != DELETE)) {
@@ -1457,6 +1447,7 @@ int accept_connection(t_binding *binding, t_config *config) {
 	pthread_attr_t      child_attr;
 	pthread_t           child_thread;
 	int                 total_conns, one;
+struct timeval timer;
 
 	if ((session = (t_session*)malloc(sizeof(t_session))) == NULL) {
 		return -1;
@@ -1514,8 +1505,20 @@ int accept_connection(t_binding *binding, t_config *config) {
 				free(session);
 				return -1;
 			}
+
+			if (config->socket_send_timeout > 0) {
+				timer.tv_sec  = config->socket_send_timeout;
+				timer.tv_usec = 0;
+				if (setsockopt(session->client_socket, SOL_SOCKET, SO_SNDTIMEO, &timer, sizeof(struct timeval)) == -1) {
+					close(session->client_socket);
+					free(session);
+					return -1;
+				}
+			}
 		}
 
+		/* Pthread initialization
+		 */
 		if (pthread_attr_init(&child_attr) != 0) {
 			log_system(session, "pthread init error");
 		} else {
