@@ -369,3 +369,12 @@ void log_close(t_host *host, bool force_close) {
 		pthread_mutex_unlock(&accesslog_mutex);
 	}
 }
+
+void log_close_for_cgi_run(t_host *host) {
+	while (host != NULL) {
+		if (*(host->access_fp) != NULL) {
+			fclose(*(host->access_fp));
+		}
+		host = host->next;
+	}
+}
