@@ -25,10 +25,14 @@ int read_mimetypes(char *configfile, t_mimetype **mime) {
 	if ((fp = fopen(configfile, "r")) == NULL) {
 		return -1;
 	}
+
 	if ((*mime = (t_mimetype*)malloc(sizeof(t_mimetype))) == NULL) {
 		fclose(fp);
 		return -1;
 	}
+	(*mime)->mimename = NULL;
+	(*mime)->extension = NULL;
+
 	line[MAX_LINE_LENGTH] = '\0';
 
 	while (fgets(line, MAX_LINE_LENGTH, fp) != NULL) {
