@@ -31,7 +31,8 @@ typedef struct type_php_cgi {
 
 typedef struct {
 	char       *pidfile;
-	char       *forks;
+	char       *children;
+	char       *max_requests;
 	t_php_cgi  *php_cgi;
 	t_keyvalue *envir;
 } t_config;
@@ -141,11 +142,19 @@ int parse_line(t_config *config, char *line) {
 				return -1;
 			}
 		} else if (strcmp(key, "forks") == 0) {
-			free(config->forks);
+			free(config->children);
 			if (str2int(value) < 1) {
 				return -1;
 			}
-			if ((config->forks = strdup(value)) == NULL) {
+			if ((config->children = strdup(value)) == NULL) {
+				return -1;
+			}
+		} else if (strcmp(key, "maxrequests") == 0) {
+			free(config->max_requests);
+			if (str2int(value) < 1) {
+				return -1;
+			}
+			if ((config->max_requests = strdup(value)) == NULL) {
 				return -1;
 			}
 		} else if (strcmp(key, "setenv") == 0) {
@@ -236,7 +245,8 @@ t_config *read_config(char *config_file) {
 	/* Default settings
 	 */
 	config->pidfile = strdup(PIDFILE_DIR"/php-fcgi.pid");
-	config->forks = strdup("3");
+	config->children = strdup("3");
+	config->max_requests = strdup("100");
 	config->php_cgi = NULL;
 	config->envir = NULL;
 
@@ -337,8 +347,8 @@ int main(int argc, char *argv[]) {
 		/* Start PHP FastCGI servers
 		 */
 		clearenv();
-		setenv("PHP_FCGI_CHILDREN", config->forks, 1);
-		setenv("PHP_FCGI_MAX_REQUESTS", "500", 1);
+		setenv("PHP_FCGI_CHILDREN", config->children, 1);
+		setenv("PHP_FCGI_MAX_REQUESTS", config->max_requests, 1);
 		env = config->envir;
 		while (env != NULL) {
 			setenv(env->key, env->value, 1);
