@@ -132,6 +132,7 @@ static t_directory *new_directory(void) {
 		directory->upload_speed        = 0;
 		directory->session_speed       = 0;
 		directory->envir_str           = NULL;
+		directory->time_for_cgi        = -1;
 		pthread_mutex_init(&(directory->client_mutex), NULL);
 
 		directory->next                = NULL;
@@ -235,6 +236,7 @@ t_config *default_config(void) {
 		config->no_extension_as    = NULL;
 		config->total_connections  = 100;
 		config->connections_per_ip = 10;
+		config->socket_send_timeout = 3;
 		config->kill_timedout_cgi  = true;
 		config->wait_for_cgi       = true;
 		config->first_host         = new_host();
@@ -622,7 +624,7 @@ int check_configuration(t_config *config) {
 
 static bool system_setting(char *key, char *value, t_config *config) {
 	char *uid, *gid, *rest;
-	t_cgi_handler *CGI;
+	t_cgi_handler *cgi;
 	t_throttle *throt;
 	int speed;
 #ifdef HAVE_COMMAND
@@ -725,16 +727,22 @@ static bool system_setting(char *key, char *value, t_config *config) {
 		}
 #endif
 	} else if (strcmp(key, "cgiextension") == 0) {
+#ifdef CIFS
+		strlower(value);
+#endif
 		if (parse_charlist(value, &(config->cgi_extension)) != -1) {
 			return true;
 		}
 	} else if (strcmp(key, "cgihandler") == 0) {
 		if (split_string(value, &value, &rest, ':') == 0) {
 			if ((*value != '\0') && (*rest != '\0')) {
-				CGI = config->cgi_handler;
+				cgi = config->cgi_handler;
 				if ((config->cgi_handler = (t_cgi_handler*)malloc(sizeof(t_cgi_handler))) != NULL) {
-					config->cgi_handler->next = CGI;
+					config->cgi_handler->next = cgi;
 					if ((config->cgi_handler->handler = strdup(value)) != NULL) {
+#ifdef CIFS
+						strlower(rest);
+#endif
 						init_charlist(&(config->cgi_handler->extension));
 						if (parse_charlist(rest, &(config->cgi_handler->extension)) != -1) {
 							return true;
@@ -850,6 +858,10 @@ static bool system_setting(char *key, char *value, t_config *config) {
 				return true;
 			}
 		}
+	} else if (strcmp(key, "socketsendtimeout") == 0) {
+		if ((config->socket_send_timeout = str2int(value)) >= 0) {
+			return true;
+		}
 	} else if (strcmp(key, "systemlogfile") == 0) {
 		if (*value == '/') {
 			if ((config->system_logfile = strdup(value)) != NULL) {
@@ -1281,6 +1293,10 @@ static bool directory_setting(char *key, char *value, t_directory *directory) {
 				return true;
 			}
 		}
+	} else if (strcmp(key, "timeforcgi") == 0) {
+		if ((directory->time_for_cgi = str2int(value)) >= 1) {
+			return true;
+		}
 	} else if (strcmp(key, "uploadspeed") == 0) {
 		if (split_string(value, &value, &maxclients, ',') == 0) {
 			if ((directory->upload_speed = str2int(value)) > 0) {
@@ -1398,6 +1414,9 @@ static bool fcgi_server_setting(char *key, char *value, t_fcgi_server *fcgi_serv
 		}
 		return true;
 	} else if (strcmp(key, "extension") == 0) {
+#ifdef CIFS
+		strlower(value);
+#endif
 		if (parse_charlist(value, &(fcgi_server->extension)) != -1) {
 			return true;
 		}
