@@ -110,6 +110,9 @@ void reset_session(t_session *session) {
 	int size;
 
 	sfree(session->file_on_disk);
+#ifdef CIFS
+	sfree(session->extension);
+#endif
 	sfree(session->local_user);
 	sfree(session->remote_user);
 	sfree(session->path_info);
@@ -202,6 +205,29 @@ void determine_request_method(t_session *session) {
 	}
 }
 
+/* Get the extension of the file to be processed
+ */
+int get_target_extension(t_session *session) {
+#ifdef CIFS
+	sfree(session->extension);
+#endif
+
+	if ((session->extension = strrchr(session->file_on_disk, '.')) != NULL) {
+		(session->extension)++;
+	} else {
+		session->extension = session->config->no_extension_as;
+	}
+
+#ifdef CIFS
+	if ((session->extension = strdup(session->extension)) == NULL) {
+		return -1;
+	}
+	strlower(session->extension);
+#endif
+
+	return 0;
+}
+
 /* Return the path of the user's homedirectory.
  */
 int get_homedir(t_session *session, char *username) {
@@ -347,6 +373,9 @@ int copy_directory_settings(t_session *session) {
 						session->host->alter_group.size = dir->alter_group.size;
 						session->host->alter_group.item = dir->alter_group.item;
 					}
+					if (dir->time_for_cgi > -1) {
+						session->host->time_for_cgi = dir->time_for_cgi;
+					}
 					break;
 				} else {
 					return 500;
