@@ -124,11 +124,10 @@ int send_file(t_session *session) {
 				if ((session->file_on_disk = (char*)malloc(strlen(session->host->imgref_replacement) + 4)) != NULL) { /* + 3 for ".gz" (gzip encoding) */
 					strcpy(session->file_on_disk, session->host->imgref_replacement);
 
-					if ((session->extension = strrchr(session->file_on_disk, '.')) != NULL) {
-						(session->extension)++;
-					} else {
-						session->extension = session->config->no_extension_as;
+					if (get_target_extension(session) == -1) {
+						return 500;
 					}
+
 					session->mimetype = get_mimetype(session->extension, session->config->mimetype);
 				} else {
 					return 500;
@@ -675,7 +674,9 @@ int execute_cgi(t_session *session) {
 		set_client_status(session, "execute_cgi()::read_from_cgi()");
 #endif
 
-		if (session->cgi_type == fastcgi) {
+		if (time(NULL) > cgi_info.deadline) {
+			cgi_result = cgi_TIMEOUT;
+		} else if (session->cgi_type == fastcgi) {
 			cgi_result = read_from_fcgi_server(session, &cgi_info);
 		} else {
 			cgi_result = read_from_cgi_process(session, &cgi_info);
@@ -845,11 +846,9 @@ int execute_cgi(t_session *session) {
 									break;
 */
 								}
-							} else {
-								if (send_buffer(session, cgi_info.input_buffer, cgi_info.input_len) == -1) {
-									retval = rs_DISCONNECT;
-									break;
-								}
+							} else if (send_buffer(session, cgi_info.input_buffer, cgi_info.input_len) == -1) {
+								retval = rs_DISCONNECT;
+								break;
 							}
 
 							in_body = true;
@@ -885,8 +884,12 @@ int execute_cgi(t_session *session) {
 		close(cgi_info.from_cgi);
 	} else {
 		close(cgi_info.to_cgi);
-		close(cgi_info.from_cgi);
-		close(cgi_info.cgi_error);
+		if (cgi_info.from_cgi != -1) {
+			close(cgi_info.from_cgi);
+		}
+		if (cgi_info.cgi_error != -1) {
+			close(cgi_info.cgi_error);
+		}
 	}
 
 	if (session->config->wait_for_cgi && (cgi_pid != -1)) {
