@@ -380,7 +380,7 @@ int check_main_config(char *config_dir) {
 	/* FastCGI Id check
 	 */
 	haystack = search_key(config, "fastcgiid");
-	needles = needle = search_key(config, "fastcgi");
+	needles = needle = search_key(config, "usefastcgi");
 	while (needle != NULL) {
 		if ((rest = strdup(needle->value)) == false) {
 			mem_error();
@@ -400,8 +400,8 @@ int check_main_config(char *config_dir) {
 	needle = config;
 	while (needle != NULL) {
 		if (strcmp(needle->key, "match") == 0) {
-			if ((item = strcasestr(needle->value, " fastcgi ")) != NULL) {
-				item += 8;
+			if ((item = strcasestr(needle->value, " usefastcgi ")) != NULL) {
+				item += 12;
 				while ((*item == ' ') && (*item != '\0')) {
 					item++;
 				}
